@@ -20,7 +20,7 @@
 #' # set up for billing
 #' con <- DBI::dbConnect(bigquery(), project = bq_test_project())
 #'
-#' shakespeare <- con %>% tbl("publicdata.samples.shakespeare")
+#' shakespeare <- con %>% tbl(I("publicdata.samples.shakespeare"))
 #' shakespeare
 #' shakespeare %>%
 #'   group_by(word) %>%
@@ -45,16 +45,28 @@ src_bigquery <- function(project, dataset, billing = project, max_pages = 10) {
 tbl.BigQueryConnection <- function(src, from, ...) {
   src <- dbplyr::src_dbi(src, auto_disconnect = FALSE)
 
+  sql <- dbplyr::sql_query_fields(src$con, from)
+  dataset <- if (!is.null(src$con@dataset)) as_bq_dataset(src$con)
+  schema <- bq_perform_query_schema(
+    sql,
+    billing = src$con@billing,
+    default_dataset = dataset
+  )
+  vars <- map_chr(schema, "[[", "name")
+
   if (utils::packageVersion("dbplyr") >= "2.4.0.9000") {
-    tbl <- dplyr::tbl(src, from = from)
+    tbl <- dplyr::tbl(src, from = from, vars = vars)
   } else {
-    tbl <- dplyr::tbl(src, from = from, check_from = FALSE)
+    tbl <- dplyr::tbl(src, from = from, vars = vars, check_from = FALSE)
   }
 
   # This is ugly, but I don't see a better way of doing this
   tb <- as_bq_table(src$con, from)
   tbl$lazy_query$is_view <- !inherits(from, "sql") &&
-    tryCatch(bq_table_meta(tb, "type")$type == "VIEW", bigrquery_notFound = function(e) TRUE)
+    tryCatch(
+      bq_table_meta(tb, "type")$type == "VIEW",
+      bigrquery_notFound = function(e) TRUE
+    )
   tbl
 }
 
@@ -62,25 +74,31 @@ tbl.BigQueryConnection <- function(src, from, ...) {
 dbplyr_edition.BigQueryConnection <- function(con) 2L
 
 # registered onLoad
-db_compute.BigQueryConnection <- function(con,
-                                          table,
-                                          sql,
-                                          ...,
-                                          overwrite = FALSE,
-                                          temporary = TRUE,
-                                          unique_indexes = list(),
-                                          indexes = list(),
-                                          analyze = TRUE,
-                                          in_transaction = FALSE) {
-
+db_compute.BigQueryConnection <- function(
+  con,
+  table,
+  sql,
+  ...,
+  overwrite = FALSE,
+  temporary = TRUE,
+  unique_indexes = list(),
+  indexes = list(),
+  analyze = TRUE,
+  in_transaction = FALSE
+) {
   if (is.null(con@dataset)) {
     destination_table <- if (!temporary) as_bq_table(con, table)
-    tb <- bq_project_query(con@project, sql, destination_table = destination_table)
+    tb <- bq_project_query(
+      con@project,
+      sql,
+      destination_table = destination_table
+    )
   } else {
     ds <- bq_dataset(con@project, con@dataset)
     destination_table <- if (!temporary) as_bq_table(con, table)
 
-    tb <- bq_dataset_query(ds,
+    tb <- bq_dataset_query(
+      ds,
       query = sql,
       destination_table = destination_table
     )
@@ -90,18 +108,19 @@ db_compute.BigQueryConnection <- function(con,
 }
 
 # registered onLoad
-db_copy_to.BigQueryConnection <- function(con,
-                                          table,
-                                          values,
-                                          ...,
-                                          overwrite = FALSE,
-                                          types = NULL,
-                                          temporary = TRUE,
-                                          unique_indexes = NULL,
-                                          indexes = NULL,
-                                          analyze = TRUE,
-                                          in_transaction = TRUE) {
-
+db_copy_to.BigQueryConnection <- function(
+  con,
+  table,
+  values,
+  ...,
+  overwrite = FALSE,
+  types = NULL,
+  temporary = TRUE,
+  unique_indexes = NULL,
+  indexes = NULL,
+  analyze = TRUE,
+  in_transaction = TRUE
+) {
   if (temporary) {
     cli::cli_abort("BigQuery does not support temporary tables")
   }
@@ -116,17 +135,36 @@ db_copy_to.BigQueryConnection <- function(con,
 # Efficient downloads -----------------------------------------------
 
 # registered onLoad
-collect.tbl_BigQueryConnection <- function(x, ...,
-                                           page_size = NULL,
-                                           max_connections = 6L,
-                                           n = Inf,
-                                           warn_incomplete = TRUE) {
 
+#' Collect a BigQuery table
+#'
+#' This collect method is specialised for BigQuery tables, generating the
+#' SQL from your dplyr commands, then calling [bq_project_query()]
+#' or [bq_dataset_query()] to run the query, then [bq_table_download()]
+#' to download the results. Thus the arguments are a combination of the
+#' arguments to [dplyr::collect()], `bq_project_query()`/`bq_dataset_query()`,
+#' and `bq_table_download()`.
+#'
+#' @inheritParams dplyr::collect
+#' @inheritParams bq_table_download
+#' @param n Maximum number of results to retrieve.
+#'   The default, `Inf`, will retrieve all rows.
+#' @param ... Other arguments passed on to
+#'   `bq_project_query()`/`bq_project_query()`
+collect.tbl_BigQueryConnection <- function(
+  x,
+  ...,
+  n = Inf,
+  api = c("json", "arrow"),
+  page_size = NULL,
+  max_connections = 6L
+) {
+  api <- check_api(api)
   check_number_whole(n, min = 0, allow_infinite = TRUE)
   check_number_whole(max_connections, min = 1)
-  check_bool(warn_incomplete)
 
   con <- dbplyr::remote_con(x)
+  billing <- con@billing
 
   if (op_can_download(x)) {
     lq <- x$lazy_query
@@ -136,7 +174,6 @@ collect.tbl_BigQueryConnection <- function(x, ...,
   } else {
     sql <- dbplyr::db_sql_render(con, x)
 
-    billing <- con@billing
     if (is.null(con@dataset)) {
       tb <- bq_project_query(billing, sql, quiet = con@quiet, ...)
     } else {
@@ -147,13 +184,28 @@ collect.tbl_BigQueryConnection <- function(x, ...,
 
   quiet <- if (n < 100) TRUE else con@quiet
   bigint <- con@bigint %||% "integer"
-  out <- bq_table_download(tb,
-    n_max = n,
-    page_size = page_size,
-    quiet = quiet,
-    max_connections = max_connections,
-    bigint = bigint
-  )
+
+  if (api == "arrow") {
+    out <- bq_table_download(
+      tb,
+      n_max = n,
+      quiet = quiet,
+      bigint = bigint,
+      billing = billing,
+      api = "arrow"
+    )
+  } else {
+    out <- bq_table_download(
+      tb,
+      n_max = n,
+      page_size = page_size,
+      quiet = quiet,
+      max_connections = max_connections,
+      bigint = bigint,
+      api = "json"
+    )
+  }
+
   dplyr::grouped_df(out, intersect(dbplyr::op_grps(x), names(out)))
 }
 
@@ -182,15 +234,27 @@ op_can_download.lazy_base_query <- function(x) {
 }
 
 query_is_head_only <- function(x) {
-  if (!inherits(x$x, "lazy_base_remote_query")) return(FALSE)
-  if (!op_can_download(x$x)) return(FALSE)
+  if (!inherits(x$x, "lazy_base_remote_query")) {
+    return(FALSE)
+  }
+  if (!op_can_download(x$x)) {
+    return(FALSE)
+  }
 
   vars_base <- dbplyr::op_vars(x$x)
-  if (!is_select_trivial(x$select, vars_base)) return(FALSE)
+  if (!is_select_trivial(x$select, vars_base)) {
+    return(FALSE)
+  }
 
-  if (!is_empty(x$where)) return(FALSE)
-  if (!is_empty(x$order_by)) return(FALSE)
-  if (!is_false(x$distinct)) return(FALSE)
+  if (!is_empty(x$where)) {
+    return(FALSE)
+  }
+  if (!is_empty(x$order_by)) {
+    return(FALSE)
+  }
+  if (!is_false(x$distinct)) {
+    return(FALSE)
+  }
 
   TRUE
 }
@@ -221,7 +285,9 @@ op_table.lazy_query <- function(x, con) NULL
 op_table.lazy_base_remote_query <- function(x, con) x$x
 #' @export
 op_table.lazy_select_query <- function(x, con) {
-  if (!query_is_head_only(x)) return(NULL)
+  if (!query_is_head_only(x)) {
+    return(NULL)
+  }
 
   op_table(x$x)
 }
@@ -247,7 +313,8 @@ sql_join_suffix.BigQueryConnection <- function(con, ...) {
 # registered onLoad
 sql_translation.BigQueryConnection <- function(x) {
   dbplyr::sql_variant(
-    dbplyr::sql_translator(.parent = dbplyr::base_scalar,
+    dbplyr::sql_translator(
+      .parent = dbplyr::base_scalar,
 
       `^` = sql_prefix("POW"),
       `%%` = sql_prefix("MOD"),
@@ -255,9 +322,15 @@ sql_translation.BigQueryConnection <- function(x) {
 
       # Coercion
       as.integer = function(x) dbplyr::build_sql("SAFE_CAST(", x, " AS INT64)"),
-      as.logical = function(x) dbplyr::build_sql("SAFE_CAST(", x, " AS BOOLEAN)"),
-      as.numeric = function(x) dbplyr::build_sql("SAFE_CAST(", x, " AS FLOAT64)"),
-      as.character = function(x) dbplyr::build_sql("SAFE_CAST(", x, " AS STRING)"),
+      as.logical = function(x) {
+        dbplyr::build_sql("SAFE_CAST(", x, " AS BOOLEAN)")
+      },
+      as.numeric = function(x) {
+        dbplyr::build_sql("SAFE_CAST(", x, " AS FLOAT64)")
+      },
+      as.character = function(x) {
+        dbplyr::build_sql("SAFE_CAST(", x, " AS STRING)")
+      },
 
       # Date/time
       Sys.date = sql_prefix("current_date"),
@@ -280,7 +353,7 @@ sql_translation.BigQueryConnection <- function(x) {
       paste0 = sql_prefix("CONCAT"),
 
       # stringr equivalents
-      str_detect =  sql_prefix("REGEXP_CONTAINS", 2),
+      str_detect = sql_prefix("REGEXP_CONTAINS", 2),
       str_extract = sql_prefix("REGEXP_EXTRACT", 2),
       str_replace = sql_prefix("REGEXP_REPLACE", 3),
 
@@ -293,13 +366,15 @@ sql_translation.BigQueryConnection <- function(x) {
         dbplyr::sql_runif(RAND(), n = {{ n }}, min = min, max = max)
       },
     ),
-    dbplyr::sql_translator(.parent = dbplyr::base_agg,
+    dbplyr::sql_translator(
+      .parent = dbplyr::base_agg,
+
       n = function() dplyr::sql("count(*)"),
 
       all = sql_prefix("LOGICAL_AND", 1),
       any = sql_prefix("LOGICAL_OR", 1),
 
-      sd =  sql_prefix("STDDEV_SAMP"),
+      sd = sql_prefix("STDDEV_SAMP"),
       var = sql_prefix("VAR_SAMP"),
       cor = dbplyr::sql_aggregate_2("CORR"),
       cov = dbplyr::sql_aggregate_2("COVAR_SAMP"),
@@ -309,11 +384,13 @@ sql_translation.BigQueryConnection <- function(x) {
         dbplyr::build_sql("APPROX_QUANTILES(", x, ", 2)[SAFE_ORDINAL(2)]")
       }
     ),
-    dbplyr::sql_translator(.parent = dbplyr::base_win,
+    dbplyr::sql_translator(
+      .parent = dbplyr::base_win,
+
       all = dbplyr::win_absent("LOGICAL_AND"),
       any = dbplyr::win_absent("LOGICAL_OR"),
 
-      sd =  dbplyr::win_recycled("STDDEV_SAMP"),
+      sd = dbplyr::win_recycled("STDDEV_SAMP"),
       var = dbplyr::win_recycled("VAR_SAMP"),
       cor = dbplyr::win_absent("CORR"),
       cov = dbplyr::win_absent("COVAR_SAMP"),
@@ -326,7 +403,8 @@ sql_translation.BigQueryConnection <- function(x) {
 }
 
 simulate_bigrquery <- function(use_legacy_sql = FALSE) {
-  new("BigQueryConnection",
+  new(
+    "BigQueryConnection",
     project = "test",
     dataset = "test",
     billing = "test",
