@@ -1,5 +1,3 @@
-context("test-bq-dataset.R")
-
 test_that("can create and delete datasets", {
   ds <- bq_dataset(bq_test_project(), random_name())
   expect_false(bq_dataset_exists(ds))
@@ -13,7 +11,7 @@ test_that("can create and delete datasets", {
 
 test_that("can update dataset metadata", {
   ds <- bq_dataset(bq_test_project(), random_name())
-  on.exit(bq_dataset_delete(ds))
+  defer(bq_dataset_delete(ds))
 
   bq_dataset_create(ds, description = "a", friendly_name = "b")
   bq_dataset_update(ds, description = "b")
@@ -27,7 +25,11 @@ test_that("by default can not delete dataset containing tables", {
   ds <- bq_test_dataset()
 
   bq_table_create(bq_table(ds, "testing"))
-  expect_error(bq_dataset_delete(ds))
+  expect_snapshot(
+    bq_dataset_delete(ds),
+    error = TRUE,
+    transform = function(x) gsub(ds$dataset, "<DATASET>", x)
+  )
 })
 
 test_that("can list tables in a dataset", {
@@ -35,6 +37,6 @@ test_that("can list tables in a dataset", {
 
   expect_equal(
     bq_dataset_tables(ds),
-    list(bq_table(ds, "mtcars"))
+    list(bq_table(ds, "mtcars"), bq_table(ds, "simple-five"))
   )
 })
